from zope.interface import implements
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

from z3c.formwidget.query.interfaces import IQuerySource
from zope.schema.vocabulary import SimpleVocabulary

from Products.CMFCore.utils import getToolByName

class CMFContentSearchSource(object):
    implements(IQuerySource)

    def __init__(self, context):
        self.context = context
        self.intid_utility = getUtility(IIntIds)

    def __contains__(self, term):
        return self.intid_utility.queryId(term) is not None

    def __iter__(self):
        return [].__iter__()

    def __len__(self):
        return 0

    def getTerm(self, obj):
        return SimpleVocabulary.createTerm(obj, self.intid_utility.getId(obj),
                                           obj.Title())

    def getTermByToken(self, value):
        return self.getTerm(self.intid_utility.getObject(int(value)))

    def search(self, query_string):
        catalog = getToolByName(self.context, 'portal_catalog')
        result = catalog(SearchableText='%s*' % query_string, sort_limit=20)
        terms = []
        for brain in result:
            try:
                term = self.getTerm(brain.getObject())
            except KeyError:
                # An object without an intid in the catalog results
                continue
            terms.append(term)
        return terms


from zope.component import getUtility
from zc.relation.interfaces import ICatalog
from z3c.relationfield.index import RelationCatalog
from zope.app.intid.interfaces import IIntIds as app_IIntIds
from zope.intid.interfaces import IIntIds
from five.intid.site import addUtility
from five.intid.intid import IntIds
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import IDynamicType

def add_relations(context):
    addUtility(context, ICatalog, RelationCatalog, ofs_name='relations',
               findroot=False)

def add_intids(context):
    # We need to explicilty use the zope.intids interface and
    # the zope.app.intids one
    addUtility(context, IIntIds, IntIds, ofs_name='intids',
               findroot=False)
    addUtility(context, app_IIntIds, IntIds, ofs_name='intids',
               findroot=False)

def register_all_content_for_intids(portal):
    """Registers all existing content with the intid utility.  This
    will not be fast."""
    cat = getToolByName(portal, 'portal_catalog', None)
    intids = getUtility(IIntIds)
    register = intids.register
    # Take advantage of paths stored in keyreferences in five.intid to optimize
    # registration
    registered_paths = dict((ref.path,None) for ref in intids.ids
                                                        if hasattr(ref, 'path'))
    # Count how many objects we register
    registered = 0
    existing = 0
    if cat is not None:
        content = cat(object_provides=IDynamicType.__identifier__)
        for brain in content:
            if brain.getPath() in registered_paths:
                existing += 1
                continue
            try:
                obj = brain.getObject()
                register(obj)
                registered += 1
            except (AttributeError, KeyError):
                pass
    return registered, existing

def installRelations(context):
    if context.readDataFile('install_relations.txt') is None:
        return
    portal = context.getSite()
    add_intids(portal)
    add_relations(portal)
    registered, existing = register_all_content_for_intids(portal)
    return ("Added relations and intid utilities.  "
            "Assigned intids to %s content objects, %s objects "
            "already had intids."%(registered, existing))



from setuptools import setup, find_packages
import os

version = '1.0b2'

setup(name='plone.app.relationfield',
      version=version,
      description="Plone support for z3c.relationfield",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='dexterity relations plone zc.relation',
      author='Alec Mitchell',
      author_email='apm13@columbia.edu',
      url='http://code.google.com/p/dexterity',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plone', 'plone.app'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'zope.interface',
          'zope.component',
          'zope.schema',
          'zope.intid',
          'five.intid >=0.4.1dev-r82705',
          'z3c.form',
          'z3c.relationfield>=0.4.2',
          'z3c.formwidget.query',
          'plone.formwidget.contenttree',
          'plone.supermodel',
          'Products.CMFCore',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from zope.component import adapts, getUtility
from zope.interface import Interface
from zope.intid.interfaces import IIntIds
from z3c.relationfield.interfaces import (
    IRelation,
    IRelationValue,
    IRelationList,
    )
from z3c.relationfield.schema import RelationChoice, RelationList
from z3c.relationfield.relation import RelationValue
from z3c.form.datamanager import AttributeField

from plone.supermodel.exportimport import BaseHandler

class RelationDataManager(AttributeField):
    """A data manager which uses the z3c.relationfield api to set
    relationships using a schema field."""
    adapts(Interface, IRelation)

    def get(self):
        """Gets the target"""
        rel = None
        try:
            rel = super(RelationDataManager, self).get()
        except AttributeError:
            # Not set yet
            pass
        if rel is not None:
            if rel.isBroken():
                # XXX: should log or take action here
                return
            return rel.to_object

    def set(self, value):
        """Sets the relationship target"""
        current = None
        try:
            current = super(RelationDataManager, self).get()
        except AttributeError:
            pass
        intids = getUtility(IIntIds)
        to_id = intids.getId(value)
        if IRelationValue.providedBy(current):
            # If we already have a relation, just set the to_id
            current.to_id = to_id
        else:
            # otherwise create a relationship
            rel = RelationValue(to_id)
            super(RelationDataManager, self).set(rel)


class RelationListDataManager(AttributeField):
    """A data manager which sets a list of relations"""
    adapts(Interface, IRelationList)

    def get(self):
        """Gets the target"""
        rel_list = []
        
        # Calling query() here will lead to infinite recursion!
        try:
            rel_list = super(RelationListDataManager, self).get()
        except AttributeError:
            rel_list = None
        
        if not rel_list:
            return []
        
        resolved_list = []
        for rel in rel_list:
            if rel.isBroken():
                # XXX: should log or take action here
                continue
            resolved_list.append(rel.to_object)
        return resolved_list

    def set(self, value):
        """Sets the relationship target"""
        value = value or []
        new_relationships = []
        intids = getUtility(IIntIds)
        for item in value:
            # otherwise create one
            to_id = intids.getId(item)
            new_relationships.append(RelationValue(to_id))
        super(RelationListDataManager, self).set(new_relationships)


# plone.supermodel schema import/export handlers

RelationChoiceHandler = BaseHandler(RelationChoice)
RelationListHandler = BaseHandler(RelationList)



from zope.interface import Interface

from plone.z3cform import layout
from z3c.form import form, field
from z3c.relationfield.schema import RelationChoice, RelationList

from plone.formwidget.autocomplete.widget import (
    AutocompleteMultiFieldWidget,
    )
from plone.formwidget.contenttree import (
    ContentTreeFieldWidget,
    ObjPathSourceBinder,
    )


class ITestForm(Interface):
    multiple = RelationList(title=u"Multiple (Relations field)",
                           required=False,
                           value_type=RelationChoice(title=u"Multiple",
                     vocabulary="plone.formwidget.relations.cmfcontentsearch"))
    single = RelationChoice(title=u"Single",
                       required=False,
                       source=ObjPathSourceBinder())


class TestForm(form.EditForm):
    fields = field.Fields(ITestForm)
    fields['multiple'].widgetFactory = AutocompleteMultiFieldWidget
    fields['single'].widgetFactory = ContentTreeFieldWidget


TestView = layout.wrap_form(TestForm)
